@@ -1,5 +1,6 @@
 import datetime
 import inspect
+import json
 import logging
 import logging.config
 import os
@@ -11,7 +12,8 @@ import git
 import orjson
 import sentry_sdk
 import structlog
-import structlog_sentry
+
+from structlog_sentry_logger import structlog_sentry
 
 
 def get_git_root() -> pathlib.Path:
@@ -33,6 +35,11 @@ ROOT_DIR = get_root_dir()
 LOG_DATA_DIR = ROOT_DIR / ".logs"
 LOG_DATA_DIR.mkdir(exist_ok=True)
 DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
+_CONFIGS = {"USE_ORJSON": True}
+
+
+def _toggle_json_library(use_orjson: bool = True) -> None:
+    _CONFIGS["USE_ORJSON"] = use_orjson
 
 
 def get_namespaced_module_name(__file__: Union[pathlib.Path, str]) -> str:
@@ -57,7 +64,7 @@ def deduce_module(prev_stack_frame: inspect.FrameInfo) -> Optional[ModuleType]:
 
 
 def get_caller_name_from_frames(stack_frames: List[inspect.FrameInfo]) -> str:
-    prev_stack_frame = stack_frames[1]
+    prev_stack_frame = stack_frames[0]
     return get_caller_name(prev_stack_frame)
 
 
@@ -155,7 +162,9 @@ def serializer(
     default: Optional[Callable[[Any], Any]] = None,
     option: Optional[int] = orjson.OPT_SORT_KEYS,
 ) -> str:
-    return orjson.dumps(*args, default=default, option=option).decode()  # type: ignore[misc]
+    if _CONFIGS["USE_ORJSON"]:
+        return orjson.dumps(*args, default=default, option=option).decode()  # type: ignore[misc]
+    return json.dumps(*args, sort_keys=True)
 
 
 def get_handlers(module_name: str) -> dict:
@@ -193,6 +202,7 @@ def set_structlog_config(timestamper: structlog.processors.TimeStamper) -> None:
         timestamper,
         structlog.processors.StackInfoRenderer(),
         structlog.processors.format_exc_info,
+        add_severity_field_from_level_if_in_cloud_environment,
     ]
     stdlib_log_compatibility_processors = [
         structlog.stdlib.add_log_level,
@@ -207,7 +217,7 @@ def set_structlog_config(timestamper: structlog.processors.TimeStamper) -> None:
         processors=(
             stdlib_log_compatibility_processors  # type: ignore[arg-type]
             + structlog_processors
-            + format_wrapper_processer  # type: ignore[operator]
+            + format_wrapper_processer  # type: ignore[arg-type,operator]
         ),
         # See [Performance](https://www.structlog.org/en/stable/performance.html)
         # for an in-depth explanation of the below settings
@@ -218,6 +228,66 @@ def set_structlog_config(timestamper: structlog.processors.TimeStamper) -> None:
     )
 
 
+def add_severity_field_from_level_if_in_cloud_environment(
+    logger: Any,  # pylint: disable=unused-argument
+    method: str,  # pylint: disable=unused-argument
+    event_dict: structlog.types.EventDict,
+) -> structlog.types.EventDict:
+    """A custom processor for structlog for Cloud Logging compatibility
+
+    Since Cloud Logging infers log levels from the `severity` key, simply duplicates
+    `level` to the `severity` field in the logger's event dictionary.
+    """
+
+    if (
+        is_cloud_logging_compatibility_mode_requested()
+        or is_probably_in_cloud_environment()
+    ):
+        cloud_logging_log_level_key, python_log_level_key = "severity", "level"
+        if cloud_logging_log_level_key in event_dict:
+            # Dogfood by instantiating a local logger with own library.
+            # Note: NO infinite loop since the below log message does *NOT* use
+            # `severity` as a key in the emitted event.
+            local_logger = get_logger()
+            local_logger.warning(
+                "Existing log value being overwritten",
+                src_key=python_log_level_key,
+                dest_key=cloud_logging_log_level_key,
+                old_value=event_dict[cloud_logging_log_level_key],
+                new_value=event_dict[python_log_level_key],
+                logger_name=logger.name,
+            )
+        event_dict[cloud_logging_log_level_key] = event_dict[python_log_level_key]
+    return event_dict
+
+
+def is_cloud_logging_compatibility_mode_requested() -> bool:
+    return "STRUCTLOG_SENTRY_LOGGER_CLOUD_LOGGING_COMPATIBILITY_MODE_ON" in os.environ
+
+
+def is_probably_in_cloud_environment() -> bool:
+    """Returns True if it is *likely* (but not guaranteed) logging is occurring in the context of a Cloud Logging environment"""
+    for env_var in [
+        # GKE
+        # There are no GKE-specific environment variable that definitively imply we are
+        # running in GKE... Falling back to detecting Kubernetes-injected environment
+        # variables since those are the only ones present in GKE pods that *could* imply
+        # we are running in GKE.
+        # Kubernetes
+        # see: https://kubernetes.io/docs/concepts/services-networking/connect-applications-service/#environment-variables
+        "KUBERNETES_SERVICE_HOST",
+        # Cloud Function
+        # see: https://cloud.google.com/functions/docs/configuring/env-var#runtime_environment_variables_set_automatically
+        "GCP_PROJECT",
+        # GAE
+        # see: https://cloud.google.com/functions/docs/configuring/env-var#runtime_environment_variables_set_automatically
+        "GOOGLE_CLOUD_PROJECT",
+    ]:
+        if env_var in os.environ:
+            return True
+    return False
+
+
 class SentryBreadcrumbJsonProcessor(structlog_sentry.SentryJsonProcessor):
 
     """
@@ -230,20 +300,15 @@ class SentryBreadcrumbJsonProcessor(structlog_sentry.SentryJsonProcessor):
 
     def __init__(  # pylint: disable=too-many-arguments
         self,
-        level: int = logging.WARNING,
         breadcrumb_level: int = logging.INFO,
+        level: int = logging.WARNING,
         active: bool = True,
         as_extra: bool = True,
-        tag_keys: Union[  # pylint: disable=unsubscriptable-object
-            List[str], str
-        ] = None,
+        tag_keys: Union[List[str], str] = None,
     ) -> None:
         self.breadcrumb_level = breadcrumb_level
         super().__init__(
-            level=level,
-            active=active,
-            as_extra=as_extra,
-            tag_keys=tag_keys,
+            level=level, active=active, as_extra=as_extra, tag_keys=tag_keys
         )
 
     @staticmethod
@@ -264,7 +329,7 @@ class SentryBreadcrumbJsonProcessor(structlog_sentry.SentryJsonProcessor):
 
     def __call__(
         self, logger: Any, method: str, event_dict: structlog.types.EventDict
-    ) -> dict:
+    ) -> structlog.types.EventDict:
         do_breadcrumb = (
             getattr(logging, event_dict["level"].upper()) >= self.breadcrumb_level
         )
